/*******************************************************************************
File:			ASCII.c

Abstract:	ASCII conversion routine function library.

Platform:	Any

Functions:	extern void	ASCIISigned(char *address, short value, char type)
		extern void	ASCIIUnsigned(char *address, unsigned short value, char type)

Version:		1.01	06/22/09	GND	Gary N. Dion
				1.02	06/22/09	GND	converted to 32 bit

*******************************************************************************/

// OS headers
#include <avr/io.h>

// General purpose include files
#include "StdDefines.h"

// App required include files
#include "ASCII.h"

// Local Function Prototypes
static uint8_t	NibbleHex(int16_t value);

/******************************************************************************/
extern void	ASCIIHex(uint8_t *address, int16_t value, uint8_t type)
/*******************************************************************************
* ABSTRACT:	Convert an 8 or 16-bit value into a 3-to-5 character null-terminated
*				ASCII	string located at "address".
*
* INPUT:		*address		Pointer to a 5-character buffer to hold the result
*				value			The value to convert into an ASCII hex string
*				type			0 = char, 1 = short
* RETURN:	None
*/
{
	uint8_t index = 0;
	if (type)									// Type is short
	{
		address[index++] = NibbleHex(value>>12);
		address[index++] = NibbleHex(value>>8);
	}

	address[index++] = NibbleHex(value>>4);
	address[index++] = NibbleHex(value);
	address[index] = 0;

	return;
}		// End ASCIIHex(...)


/******************************************************************************/
static uint8_t	NibbleHex(int16_t value)
/*******************************************************************************
* ABSTRACT:	Convert a 4-bit value into an ASCII character.
*
* INPUT:		value			The value to convert into an ASCII character
* RETURN:	char			The ASCII letter corresponding to the nibble
*/
{
	value &= 0x000F;
	if (value>9)								// Value is A-F
	{
		return (value - 10) + 'A';
	}
	else
	{
		return value + '0';
	}

}		// End NibbleHex(...)


/******************************************************************************/
extern void	ASCIISigned(uint8_t *address, int32_t value, uint8_t type)
/*******************************************************************************
* ABSTRACT:	Convert an 8, 16, or 32-bit value into a 4-to-7 character 
*				null-terminated ASCII string located at "address".
*
* INPUT:		*address		Pointer to a 7-character buffer to hold the result
*				value			The signed value to convert into an ASCII string
*				type			0 = char, 1 = short, 2 = long
* RETURN:	None
*/
{
	if (value < 0)								// value passed is negative
	{
		address[0] = '-';
		value = -value;
	}
	else
	{
		address[0] = '+';
	}

	ASCIIUnsigned(address + 1, (uint32_t) value, type);

	return;

}		// End ASCIISigned(...)


/******************************************************************************/
extern void	ASCIIUnsigned(uint8_t *address, uint32_t value, uint8_t type)
/*******************************************************************************
* ABSTRACT:	Convert an 8, 16, or 32-bit value into a 4-to-7 character 
*				null-terminated ASCII string located at "address".
*
* INPUT:		*address		Pointer to a 6-character buffer to hold the result
*				value			The unsigned value to convert into text
*				type			0 = char, 1 = short, 2 = long
* RETURN:	None
*/
{
	uint8_t leading = FALSE;
	uint8_t index = 0;
	uint8_t count = 0;

	if (type == 2)										// Type is long
	{
		while (value >= 1000000000)
		{
			value -= 1000000000;
			count++;
		}
		if (count)
		{
			address[index++] = count + '0';
			leading = TRUE;
		}
#ifdef LEADINGSPACES
		else address[index++] = ' ';
#endif
#ifdef LEADINGZEROS
		else address[index++] = '0';
#endif

		count = 0;
		while (value >= 100000000)
		{
			value -= 100000000;
			count++;
		}

		if (count || leading)
		{
			address[index++] = count + '0';
			leading = TRUE;
		}
#ifdef LEADINGSPACES
		else address[index++] = ' ';
#endif
#ifdef LEADINGZEROS
		else address[index++] = '0';
#endif

		count = 0;
		while (value >= 10000000)
		{
			value -= 10000000;
			count++;
		}

		if (count || leading)
		{
			address[index++] = count + '0';
			leading = TRUE;
		}
#ifdef LEADINGSPACES
		else address[index++] = ' ';
#endif
#ifdef LEADINGZEROS
		else address[index++] = '0';
#endif

		count = 0;
		while (value >= 1000000)
		{
			value -= 1000000;
			count++;
		}

		if (count || leading)
		{
			address[index++] = count + '0';
			leading = TRUE;
		}
#ifdef LEADINGSPACES
		else address[index++] = ' ';
#endif
#ifdef LEADINGZEROS
		else address[index++] = '0';
#endif

		count = 0;
		while (value >= 100000)
		{
			value -= 100000;
			count++;
		}

		if (count || leading)
		{
			address[index++] = count + '0';
			leading = TRUE;
		}
#ifdef LEADINGSPACES
		else address[index++] = ' ';
#endif
#ifdef LEADINGZEROS
		else address[index++] = '0';
#endif

	}
	
	if (type)										// Type is short
	{
		count = 0;
		while (value >= 10000)
		{
			value -= 10000;
			count++;
		}
		if (count)
		{
			address[index++] = count + '0';
			leading = TRUE;
		}
#ifdef LEADINGSPACES
		else address[index++] = ' ';
#endif
#ifdef LEADINGZEROS
		else address[index++] = '0';
#endif

		count = 0;
		while (value >= 1000)
		{
			value -= 1000;
			count++;
		}

		if (count || leading)
		{
			address[index++] = count + '0';
			leading = TRUE;
		}
#ifdef LEADINGSPACES
		else address[index++] = ' ';
#endif
#ifdef LEADINGZEROS
		else address[index++] = '0';
#endif

	}

	count = 0;
	while (value >= 100)
	{
		value -= 100;
		count++;
	}

	if (count || leading)
	{
		address[index++] = count + '0';
		leading = TRUE;
	}
#ifdef LEADINGSPACES
	else address[index++] = ' ';
#endif
#ifdef LEADINGZEROS
	else address[index++] = '0';
#endif

	count = 0;
	while (value >= 10)
	{
		value -= 10;
		count++;
	}

	if (count || leading)
	{
		address[index++] = count + '0';
		leading = TRUE;
	}
#ifdef LEADINGSPACES
	else address[index++] = ' ';
#endif
#ifdef LEADINGZEROS
	else address[index++] = '0';
#endif

	address[index++] = value + '0';
	address[index] = 0;

	return;

}		// End ASCIIUnsigned(...)
