/*******************************************************************************
File:			Control.c

Abstract:	Control subsystem function library.

Platform:	ATmega8

Functions:
extern void	CtrlInit(void);
extern void	CtrlSchedule(uint8_t schedule);
extern void	CtrlACon(void);
extern void	CtrlACoff(void);
extern void	CtrlACmode(void);
extern void	CtrlACpower(void);
extern void	CtrlACstatus(void);

Versions:	06/03/16	GND	Gary N. Dion
*******************************************************************************/

// OS headers
#include <avr/io.h>

// General purpose include files
#include "StdDefines.h"

// App required include files
#include	"Main.h"
#include	"ASCII.h"
#include	"Control.h"
#include "Serial.h"

// Local variables

uint8_t AC_POWER[70] = {0x69, 0x00, 0xff, 0xB0, 0x18, 0x40, 0x18, 0x14, 0x18, 
				0x14, 0x18, 0x14, 0x18, 0x14, 0x18, 0x14, 0x18, 0x14, 0x18, 0x40,
				0x18, 0x14, 0x18, 0x40, 0x18, 0x40, 0x18, 0x14, 0x18, 0x14, 0x18,
				0x40, 0x18, 0x40, 0x18, 0x14, 0x18, 0x40, 0x18, 0x14, 0x18, 0x14,
				0x18, 0x14, 0x18, 0x14, 0x18, 0x14, 0x18, 0x14, 0x18, 0x40, 0x18,
				0x14, 0x18, 0x40, 0x18, 0x40, 0x18, 0x40, 0x18, 0x40, 0x18, 0x40,
				0x18, 0x40, 0x18, 0x14, 0x18, 0xFF};
						
uint8_t AC_MODE[70] =  {0x69, 0x00, 0xff, 0xB0, 0x18, 0x40, 0x18, 0x14, 0x18, 
				0x14, 0x18, 0x14, 0x18, 0x14, 0x18, 0x14, 0x18, 0x14, 0x18, 0x40,
				0x18, 0x14, 0x18, 0x40, 0x18, 0x40, 0x18, 0x14, 0x18, 0x14, 0x18,
				0x40, 0x18, 0x40, 0x18, 0x14, 0x18, 0x40, 0x18, 0x40, 0x18, 0x14,
				0x18, 0x40, 0x18, 0x40, 0x18, 0x14, 0x18, 0x14, 0x18, 0x40, 0x18,
				0x14, 0x18, 0x14, 0x18, 0x40, 0x18, 0x14, 0x18, 0x14, 0x18, 0x40,
				0x18, 0x40, 0x18, 0x14, 0x18, 0xFF};

uint8_t desired_mode = OFF;				// Either ON or OFF
uint8_t current_mode = OFF;				// Either ON or OFF
uint8_t on_sequence_state = 0;
uint8_t off_sequence_state = 0;

uint8_t ON_CMD_REC[] =  {"On Command Received"};
uint8_t OFF_CMD_REC[] = {"Off Command Received"};
uint8_t MODE_SENT[] =   {"Sent MODE code"};
uint8_t POWER_SENT[] =  {"Sent POWER code"};
uint8_t FAN_MODE[] = 	{"FAN mode entered"};
uint8_t DESIRED_MODE[] ={"Desired mode = "};
uint8_t CURRENT_MODE[] ={"Current mode = "};
uint8_t LIGHT_VALUE[] = {"Light value = "};
uint8_t ON_TXT[] = {"ON"};
uint8_t OFF_TXT[] = {"OFF"};
uint8_t FAILED[] = {"Failed to control unit"};


/******************************************************************************/
extern void		CtrlInit(void)
/*******************************************************************************
* ABSTRACT:	Initializes all I/O control registers.
*
* INPUT:		None
* RETURN:	Void
*/
{
	if (ADCGet(0) < 200) current_mode = ON;	// Set depending on state of LED
	else current_mode = OFF;
	desired_mode = current_mode;					// Don't change if already on
	
	return;

}		// End CtrlRegInit(...)


/******************************************************************************/
extern void	CtrlSecond(void)
/*******************************************************************************
* ABSTRACT:	This code is called at 1Hz.
*
* INPUT		None
* RETURN:	Void
*/
{
	if (ADCGet(0) < 200) current_mode = ON;	// Set depending on state of LED

	if ((desired_mode == ON) && (current_mode == OFF))
	{
		switch(on_sequence_state)
		{
			case (0):
				SerTxString(ON_CMD_REC, 19, 0); 
				SerTxByte('.', 0); SerTxByte(0x0D, 0); SerTxByte(0x0A, 0);
				break;
				
			case (1):
			case (5):
			case (10):
			case (15):
			case (20):
			case (25):
				CtrlACpower();
				break;
				
			case (29):
				SerTxString(FAILED, 22, 0); 
				SerTxByte('.', 0); SerTxByte(0x0D, 0); SerTxByte(0x0A, 0);
				break;

			default:
				CtrlACmode();
				break;
		}

		if (on_sequence_state < 30) on_sequence_state += 1;

	}

	return;

}		// End CtrlSchedule(...)


/******************************************************************************/
extern void	CtrlACon(void)
/*******************************************************************************
* ABSTRACT:	Meant to turn the Air Conditioner on and set it to cool
*
* INPUT:		None
* RETURN:	Void
*/
{
	desired_mode = ON;						// Either ON or OFF
	on_sequence_state = 0;					// Reset ON timer
//	MainSecondsReset();						// Reset timer overflow in Main
	return;

}		// End CtrlACon(...)


/******************************************************************************/
extern void	CtrlACoff(void)
/*******************************************************************************
* ABSTRACT:	Meant to turn the Air Conditioner to fan and eventually off
*
* INPUT:		None
* RETURN:	Void
*/
{
	desired_mode = OFF;						// Either ON or OFF
	off_sequence_state = 0;					// Reset OFF timer
//	MainSecondsReset();						// Reset timer overflow in Main
	
	return;

}		// End CtrlACoff(...)


/******************************************************************************/
extern void	CtrlACmode(void)
/*******************************************************************************
* ABSTRACT:	Send the command representing the mode button
*
* INPUT:		None
* RETURN:	Void
*/
{
	uint8_t index;								// Used to index the burst sequence

	for (index = 0; index < 70; index +=2)	// 35 Groups of IR pulses
	{
		TCCR1A |= (1<<COM1A1);					// Enable the IR pulses
//		MainIRpulses(AC_MODE[index]);			// Wait
		TCCR1A &= ~(1<<COM1A1);					// Disable the IR pulses
//		MainIRpulses(AC_MODE[index+1]);		// Wait again
	}
	
	SerTxString(MODE_SENT, 17, 0); 
	SerTxByte('.', 0); SerTxByte(0x0D, 0); SerTxByte(0x0A, 0);
	
	return;

}		// End CtrlACmode(...)


/******************************************************************************/
extern void	CtrlACpower(void)
/*******************************************************************************
* ABSTRACT:	Send the command representing the power button
*
* INPUT:		None
* RETURN:	Void
*/
{
	uint8_t index;								// Used to index the burst sequence

	for (index = 0; index < 70; index +=2)	// 35 Groups of IR pulses
	{
		TCCR1A |= (1<<COM1A1);					// Enable the IR pulses
//		MainIRpulses(AC_POWER[index]);		// Wait
		TCCR1A &= ~(1<<COM1A1);					// Disable the IR pulses
//		MainIRpulses(AC_POWER[index+1]);		// Wait again
	}

	SerTxString(POWER_SENT, 18, 0);
	SerTxByte('.', 0); SerTxByte(0x0D, 0); SerTxByte(0x0A, 0);

	return;

}		// End CtrlACpower(...)


/******************************************************************************/
extern void	CtrlACstatus(void)
/*******************************************************************************
* ABSTRACT:	Return status of the Air Conditioner
*
* INPUT:		None
* RETURN:	Void
*/
{
	uint8_t	tempstr[5];						// UTC time in HHMMSS format
	uint16_t	analog;							// Value read from the ADC

	analog =	ADCGet(0);
// 255 with light off.  131 with light on!

	ASCIIUnsigned (tempstr, analog, 1);

	SerTxString(CURRENT_MODE, 18, 0);
	if (current_mode == ON) SerTxString(ON_TXT, 2, 0);
	else SerTxString(OFF_TXT, 3, 0);

	SerTxByte('.', 0); SerTxByte(0x0D, 0); SerTxByte(0x0A, 0);
	
	SerTxString(DESIRED_MODE, 18, 0);
	if (desired_mode == ON) SerTxString(ON_TXT, 2, 0);
	else SerTxString(OFF_TXT, 3, 0);
	
	SerTxByte('.', 0); SerTxByte(0x0D, 0); SerTxByte(0x0A, 0);

	SerTxString(LIGHT_VALUE, 17, 0);
	SerTxString(tempstr, 5, 0);
	SerTxByte('.', 0); SerTxByte(0x0D, 0); SerTxByte(0x0A, 0);

	return;

}		// End CtrlACstatus(...)



