/*******************************************************************************
File:			LCD.c

 				LCD subsystem function library.

Functions:	extern void	LCDInit(void);
				extern void LCDRedraw(void);
				extern void	LCDProcesses(void);
				extern void LCDCursor(char position, char enable, char blink);
static void LCDCustom(char character, char one, char two, char three, char four,
											char five, char six, char seven, char eight);
extern void LCDsendString(char *addr, unsigned char position, char chars);
extern void LCDsendFLASHString(char *addr, unsigned char position, char chars);
extern void LCDsendEEPROMString(char *addr, unsigned char position, char chars);
			static void	PutLCDByte(unsigned char btye, unsigned char type,
																		unsigned char controller);
			static char	GetLCDByte(unsigned char type, unsigned char controller);

Versions:	1.00	04/19/05	GND	Gary N. Dion
				2.00	05/16/07	GND	Revamped for new cursor behavior
				3.00	07/01/12	GND	Multiple geometries now (16x2 and 20x4)
				3.01	07/01/12	GND	Added simple function for custom characters
*******************************************************************************/

// OS headers
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>

// General purpose include files
#include "StdDefines.h"

// App required include files
#include "LCD.h"

#define	LCD_RS			(0x08)			// Register select line (high for DATA)
#define	LCD_READ			(0x10)			// Read/_Write line for LCD controller
#define	LCD_EN			(0x20)			// Clock Enable line for LCD controller

#define	DATA				(0x01)			// Tells PutLCDByte byte is data
#define	INST				(0x00)			// Tells PutLCDByte byte is data
#define	LCD_CLR			(0x01)			// LCD clear bit
#define	LCD_BUSY			(0x80)			// LCD busy flag bit
#define	LCD_ON			(0x0C)			// Display ON instruction
#define	CURSOR_ON		(0x02)			// Cursor ON bit
#define	CURSOR_BLINK	(0x01)			// Cursor BLINK bit

uint8_t update_screen;							// Set by LCDUpdate
uint8_t cursor_pos;								// Position of the cursor on the screen
uint8_t cursor_on;								// Set if the cursor is to be visible
uint8_t cursor_blink;							// Set if the cursor is to be blinking

uint8_t LCD_array[SCREEN_SIZE];	// Display RAM, refreshed on command

static void	PutLCDByte(uint8_t btye, uint8_t type);
static uint8_t	GetLCDByte(uint8_t type);

/******************************************************************************/
extern void	LCDInit(void)
/*******************************************************************************
* ABSTRACT:	This function initializes the LCD controller(s).
*
* INPUT:		None
* RETURN:	None
*/
{
	uint8_t i;

	for (i = 0; i < SCREEN_SIZE ; i++)		// Initialize screen character buffer
		LCD_array[i] = ' ';						// Just fill with spaces

	while (GetLCDByte(INST) & LCD_BUSY);	// Wait for busy flag to clear
	PutLCDByte (0x28, INST);					// Command 4-bit mode, 2-lines
	while (GetLCDByte(INST) & LCD_BUSY);	// Do it twice!
	PutLCDByte (0x28, INST);					// Trust me, you need to.

	while (GetLCDByte(INST) & LCD_BUSY);	// Wait for busy flag to clear
	PutLCDByte (LCD_ON, INST);					// Turn on the display
	while (GetLCDByte(INST) & LCD_BUSY);	// Wait for busy flag to clear
	PutLCDByte (LCD_CLR, INST);				// Clear the display

//	LCDCursor (0, 1, 1);


#ifdef custom_characters
	LCDCustom(0,31,31,31,31,31,31,31,31);  
	LCDCustom(1,0,31,31,31,31,31,31,31);
	LCDCustom(2,0,0,31,31,31,31,31,31);
	LCDCustom(3,0,0,0,31,31,31,31,31);
	LCDCustom(4,0,0,0,0,31,31,31,31);
	LCDCustom(5,0,0,0,0,0,31,31,31);
	LCDCustom(6,0,0,0,0,0,0,31,31);
	LCDCustom(7,0,0,0,0,0,0,0,31);
#endif
	
	return;
	
}		// End LCDInit(...)


/******************************************************************************/
extern void LCDRedraw(void)
/*******************************************************************************
* ABSTRACT:	This function simply initiates a screen redraw by setting a flag.
*
* INPUT:		None
* RETURN:	None
*/
{
	update_screen = TRUE;
	
	return;
	
}		// End LCDRedraw(...)


/******************************************************************************/
extern void	LCDProcesses(void)
/*******************************************************************************
* ABSTRACT:	Called by main.c during idle time, this code first checks to see if
*				a refresh is already in process.  If so, it checks the busy pin of
*				each controller and sends a character if it is idle.  If there is
*				no refresh in progres, if looks at the update flag set by LCDRedraw.
*				If the flag has been set, this function clears it and restarts the
*				redraw process.  On completion of a screen refresh, the cursor is
*				placed in the position determined by LCDcursor.
*
* INPUT:		None
* RETURN:	None
*/
{
	uint8_t temp_char;
	static uint8_t refreshing;
	static uint8_t just_sent_newline;
	static uint8_t screen_index;

	if (!(refreshing))						// If no update refresh in progress
	{
		if (update_screen)					// Check for a request to update screen
		{
			update_screen = FALSE;			// Clear request
			refreshing = TRUE;				// Set the update flag
			screen_index = 0;					// Initialize screen buffer index
			just_sent_newline = FALSE;		// We are starting from a new line
		}
		else
		{
			if (!(GetLCDByte(INST) & LCD_BUSY))// If LCD controller is idle
			{
				temp_char = LCD_ON;				// Display on control
				if (cursor_on)						// Based on cursor status
					temp_char += CURSOR_ON;		// set the cursor bits
				if (cursor_blink)
					temp_char += CURSOR_BLINK;
				PutLCDByte (temp_char, INST);	// Display and cursor command
			}

		}		// End else

	}		// End if
	else												// Update in progress
	{
		if (!(GetLCDByte(INST) & LCD_BUSY))	// If LCD controller isn't busy
		{
			if (just_sent_newline)				// If we just began a new line
			{
				temp_char = LCD_array[screen_index++ - 1];
				PutLCDByte (temp_char, DATA);	// Write character to LCD
				just_sent_newline = FALSE;
			}
			else										// We might be at the end of a line
			{
				switch(screen_index)				// Check to see if we need a newline
				{
					case(0):
						PutLCDByte (LCD_ON, INST);	// Top left corner of screen
						screen_index++;
						just_sent_newline = FALSE;
						break;

					case(1):
						PutLCDByte (0x80, INST);	// Top left corner of screen
						just_sent_newline = TRUE;
						break;
#ifdef SCREEN_16x2
					case(17):
						PutLCDByte (0xC0, INST);	// Beginning of second line
						just_sent_newline = TRUE;
						break;

					case(33):
						PutLCDByte (0x80 + cursor_pos, INST);	// Place cursor
						refreshing = FALSE;			// After char 32, we are done!
						break;
#endif
#ifdef SCREEN_20x4
					case(21):
						PutLCDByte (0xC0, INST);	// Beginning of second line
						just_sent_newline = TRUE;
						break;

					case(41):
						PutLCDByte (0x94, INST);	// Beginning of third line
						just_sent_newline = TRUE;
						break;

					case(61):
						PutLCDByte (0xD4, INST);	// Beginning of fourth line
						just_sent_newline = TRUE;
						break;

					case(81):
						PutLCDByte (0x80 + cursor_pos, INST);	// Place cursor
						refreshing = FALSE;				// After char 79, we are done!
						break;
#endif
					default:								// No newline, so send a character
						temp_char = LCD_array[screen_index++ - 1];
						PutLCDByte (temp_char, DATA);	// Write character to LCD
						break;
				}		// End switch

			}		// End else

		}		// End if

	}		// End else

	return;
	
}		// End LCDProcesses(...)


/******************************************************************************/
extern void LCDCursor(uint8_t position, uint8_t enable, uint8_t blink)
/*******************************************************************************
* ABSTRACT:	This function controls the location and behavior of the cursor.
*
* INPUT:		position		Where to place the cursor
*				enable		Wether the cursor is on or off (0,1)
*				blink			Wether the cursor is blinking or not (0,1)
* RETURN:	None
*/
{
	cursor_pos = position;
	cursor_on = enable;
	cursor_blink = blink;
	
	return;
	
}		// End LCDCursor(...)


/******************************************************************************/
extern void LCDCustom(uint8_t character, uint8_t one, uint8_t two, 
							uint8_t three, uint8_t four, uint8_t five, uint8_t six, 
							uint8_t seven, uint8_t eight)
/*******************************************************************************
* ABSTRACT:	This function allows one to easily add custom characters.
*				There are eight characters in the RAM that we can use
*				A nice tool is found here: http://www.quinapalus.com/hd44780udg.html
*
* INPUT:		character	Character address we want to customize
*				one-eight	each byte is one line of the custom character
* RETURN:	None
*/
{
	while (GetLCDByte(INST) & LCD_BUSY);	// Wait for busy flag to clear
	PutLCDByte (0x40 + (character<<3), INST);	// Select the character to program

	while (GetLCDByte(INST) & LCD_BUSY);	// Wait for busy flag to clear
	PutLCDByte (one, DATA);	// Write character to LCD

	while (GetLCDByte(INST) & LCD_BUSY);	// Wait for busy flag to clear
	PutLCDByte (two, DATA);	// Write character to LCD

	while (GetLCDByte(INST) & LCD_BUSY);	// Wait for busy flag to clear
	PutLCDByte (three, DATA);	// Write character to LCD

	while (GetLCDByte(INST) & LCD_BUSY);	// Wait for busy flag to clear
	PutLCDByte (four, DATA);	// Write character to LCD

	while (GetLCDByte(INST) & LCD_BUSY);	// Wait for busy flag to clear
	PutLCDByte (five, DATA);	// Write character to LCD

	while (GetLCDByte(INST) & LCD_BUSY);	// Wait for busy flag to clear
	PutLCDByte (six, DATA);	// Write character to LCD

	while (GetLCDByte(INST) & LCD_BUSY);	// Wait for busy flag to clear
	PutLCDByte (seven, DATA);	// Write character to LCD

	while (GetLCDByte(INST) & LCD_BUSY);	// Wait for busy flag to clear
	PutLCDByte (eight, DATA);	// Write character to LCD

	return;
	
}		// End LCDCursor(...)


/******************************************************************************/
extern void LCDsendString(uint8_t *addr, uint8_t position, uint8_t chars)
/*******************************************************************************
* ABSTRACT:	This function sends a null-terminated string to the LCD buffer,
*				starting at coordinate 'position'.
*
* INPUT:		*szString	Pointer to string to send
*				position		Location in the buffer to start printing the string
*				chars			Maximum number of characters to copy
* RETURN:	None
*/
{
	while ((*addr != 0) && (chars--))
	{
		if (position > SCREEN_SIZE) position = 0;	// No buffer overflow!
		LCD_array[position++] = (*addr);
		addr++;
	}

	return;
	
}		// End LCDsendString(...)


/******************************************************************************/
extern void LCDsendFLASHString(const char *addr,
													uint8_t position, uint8_t chars)
/*******************************************************************************
* ABSTRACT:	This function sends a null-terminated string located in program
*				space to the LCD buffer, starting at coordinate 'position'.
*
* INPUT:		*szString	Pointer to string to send
*				position		Location in the buffer to start printing the string
*				chars			Maximum number of characters to copy
* RETURN:	None
*/
{
	uint8_t temp_char;

	temp_char = pgm_read_byte(addr++);
	while ((temp_char) && (chars--))
	{
		if (position > SCREEN_SIZE) position = 0;	// No buffer overflow!
		LCD_array[position++] = temp_char;
		temp_char = pgm_read_byte(addr++);
	}

	return;
	
}		// End LCDsendString_P(...)


/******************************************************************************/
extern void LCDsendEEPROMString(uint16_t addr,
													uint8_t position, uint8_t chars)
/*******************************************************************************
* ABSTRACT:	This function sends a null-terminated string found in EEPROM
*				at the address given to the LCD, starting at coordinate 'position'.
*
* INPUT:		address		Starting address for the string
*				position		Location in the buffer to start printing the string
*				chars			Maximum number of characters to copy
* RETURN:	None
*/
{
	uint8_t temp_char;

	temp_char = eeprom_read_byte ((uint8_t *)(addr));
	while ((temp_char) && (chars--))
	{
		if (position == SCREEN_SIZE) position = 0;	// No buffer overflow!
		LCD_array[position++] = temp_char;
		temp_char = eeprom_read_byte ((uint8_t *)(++addr));
	}

	return;
	
}		// End LCDsendEEPROMString(...)


/******************************************************************************/
static void	PutLCDByte(uint8_t byte, uint8_t type)
/*******************************************************************************
* ABSTRACT:	Send a byte to the LCD controller
*
* INPUT:		byte			The value of the byte to be sent
*				type			0 if instruction, 1 if data
* RETURN:	None
*/
{
	DDRC |= 0x0F;								// Data pins are outputs
	PORTB &= ~LCD_READ;						// Lower read line (low means write)
	if (type)									// Is type data?
	{
		PORTB |= LCD_RS;						// Then set the data bit
	}
	else
	{
		PORTB &= ~LCD_RS;						// else clear the bit
	}
	
	PORTC = ((byte >> 4) & 0x0F);			// Put lower 4 bits on data bus
	PORTB |= LCD_EN;							// Raise EN to wake up coan ntroller
	PORTB &= ~LCD_EN;							// Drop EN to latch data
	PORTC = (byte & 0x0F);					// Put upper 4 bit on data bus
	PORTB |= LCD_EN;							// Raise EN to wake up controller
	PORTB &= ~LCD_EN;							// Drop EN to latch data
	PORTC = 0x10;									// Clear the data port
	DDRC &= ~0x0F;								// Data pins are inputs

	return;

}		// End PutLCDByte(...)


/******************************************************************************/
static uint8_t	GetLCDByte(uint8_t type)
/*******************************************************************************
* ABSTRACT:	Read a byte from the LCD controller
*
* INPUT:		type			0 if instruction, 1 if data
* RETURN:	byte			The byte read from the LCD controller
*/
{
	uint8_t byte;

	DDRC &= ~0x0F;								// Data pins are inputs
	PORTC |= 0x1F;								// Enable pullups
	PORTB |= LCD_READ;						// High on read line means read
	if (type)									// Is type data?
	{
		PORTB |= LCD_RS;						// Then set the data bit
	}
	else
	{
		PORTB &= ~LCD_RS;						// else clear the bit
	}
	
	PORTB |= LCD_EN;							// Raise EN to wake up controller
	PORTB |= LCD_EN;							// Wait for data lines to settle
	byte = PINC << 4;							// Grab upper four bits of data
	PORTB &= ~LCD_EN;							// Drop the EN line
	PORTB |= LCD_EN;							// Raise EN to wake up controller again
	PORTB |= LCD_EN;							// Wait for data lines to settle
	byte |= PINC & 0x0F;						// Grab lower four bits of data
	PORTB &= ~LCD_EN;							// Drop the EN line

	return byte;
	
}		// End GetLCDByte(...)

