/*******************************************************************************
File:			Main.c

				IC-3200A add-on display program.

Functions:	
extern int	main(void)
SIGNAL(TIMER0_OVF_vect)
SIGNAL(TIMER1_CAPT_vect)
SIGNAL(TIMER2_OVF_vect)

Version:		1.00	07/31/16	GND	Created
*******************************************************************************/

// OS headers
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>

// General purpose include files
#include "StdDefines.h"

// App required include files
#include "Main.h"
#include "ASCII.h"
#include "Serial.h"
#include "LCD.h"

#define	WatchdogReset() asm("wdr")		// Macro substitution to kick the dog

// Local variables
volatile uint8_t data_bit = 128;
volatile uint8_t received_byte = 0;		// byte clocked from IC-3200A
volatile	uint8_t second_fractions = 0;	// 0-49: fractions of a second
volatile	uint8_t seconds = 0;				// Seconds in transmit timer
volatile	uint8_t minutes = 0;				// Minutes in transmit timer

// These are the tones used in the IC-3200 based on displayed number
uint16_t tones[39] = {0, 670, 719, 744, 770, 797, 825, 854, 885, 915, 948, 974,
							1000, 1035, 1072, 1109, 1148, 1188, 1230, 1273, 1318, 1365, 
							1413,	1462, 1514, 1567, 1622, 1679, 1738, 1799, 1862, 1928, 
							2035, 2107,	2181, 2251, 2336, 2418, 2503};

/******************************************************************************/
extern int main(void)
/*******************************************************************************
* ABSTRACT:	Main program entry function.
*
* INPUT:		None
* OUTPUT:	Void
*/
{
	uint8_t bytes = 0;						// Start with no bytes received
	uint32_t received_word = 0;			// 32-bit word clocked from IC-3200A
	uint8_t tempstr[11];						// String to hold some numbers to print
	uint8_t transmitting = FALSE;			// True during transmit
	uint8_t last_seconds = 0;				// Used to see if seconds has changed

// PORT C - LCD display register
// Bit/Pin 3 = Data bus line 7 (DB7)
// Bit/Pin 2 = Data bus line 6 (DB6)
// Bit/Pin 1 = Data bus line 5 (DB5)
// Bit/Pin 0 = Data bus line 4 (DB4)
	PORTC = 0x10;
	DDRC  = 0x1F;

// PORT B - LCD enable pin
// Bit/Pin 5 = Enable line for the LCD
// Bit/Pin 4 = R/W: O = Write to LCD, 1 = Read from LCD
// Bit/Pin 3 = RS: O = Instruction input, 1 = Data input
	PORTB = 0x00;
	DDRB  = 0x38;
		
/* PORT B - Input capture (clock) and data in from the IC-3200A
	Bit/Pin 1 (in) Data line
	Bit/Pin 0 (in) ICP1: Clock line */
	PORTB = 0x00;								// No pullups necessary
	DDRB |= 0x04;								// All inputs
		
	//Initialize various system functions
	SerInit();									// Ser is serial functions
	LCDInit();									// LCD is for display fuctions
		
	//	Initialize the 8-bit Timer0 to clock at 31.25 kHz @ 8Mhz.
	TCCR0 = 0x04;								// Timer0 prescale of 256
//	TIMSK |= 1<<TOIE0;						// Enable Timer0 interrupt

	// Use Input Capture to catch Clock on ICP1.  Data is valid on rising edge.
	TCCR1B = (1<<ICNC1) | (0<<ICES1);	// Turn on filter, rising edge trigger
	TIMSK |= (1<<TICIE1);					// Input Capture Interrupt Enable

	//	Initialize the 8-bit Timer2 to clock at 7.8125 kHz @ 8Mhz.
	TCCR2 = 0x07;								// Timer2 prescale of 1024
	TIMSK |= 1<<TOIE2;						// Enable Timer2 interrupt

	// Enable the watchdog timer
//	WDTCR	= (1<<WDCE) | (1<<WDE);			// Wake-up the watchdog register
//	WDTCR	= (1<<WDE) | 7;					// Enable and timeout around 2.1s

	// Enable interrupts
	sei();

//	WatchdogReset();							// Reset watchdog

	// Initialization complete - system ready.  Run program loop indefinitely.
	while(TRUE)
	{
//		WatchdogReset();						// Reset watchdog
				
		if (!(data_bit))						// Just received one byte
		{
			TCNT0 = 240;						// Reset timer for a 2ms timeout
			data_bit = 128;					// Reset byte index
			received_word = (received_word<<8) + received_byte;
			received_byte = 0;				// Prepare byte for next receive
			bytes++;								// Remember 
		}

		if (!(TCNT0))							// Timer just rolled over
		{
			TCNT0 = 1;							// Change timer value to clear event
			if (bytes == 1)					// he radio just changed CTCSS tone
			{
				received_word &= 63;			// Only seven bits are valid
				if (received_word > 38) received_word = 0;	// Only know 39 tones
				received_word = tones[received_word];			// So look up the tone
				ASCIIUnsigned (tempstr, received_word, 1);	// And convert to text
//				SerTxByte(0x0A,0);
//				SerTxByte(0x0D,0);
//				SerTxByte('t',0);				// Send tone to serial port for debug
//				SerTxByte('=',0);
//				SerTxString(tempstr, 5, 0);	
				LCDsendFLASHString(PSTR("Tone "),18,5);// Print tone to screen
				LCDsendString((tempstr + 1), 23, 3);	// First three digits
				LCDsendFLASHString(PSTR("."),26,1);		// Decimal
				LCDsendString((tempstr + 4), 27, 1);	// Last digit
				LCDsendFLASHString(PSTR("Hz"),28,2);	// Units
				LCDRedraw();					// And tell code to redraw screen
			}

			if (bytes == 3)					// The radio just changed frequency
			{ 
				if (received_word < 27000)	// This means VHF Receive
				{
					transmitting = FALSE;	// Make note we're not transmiting 
					LCDsendFLASHString(PSTR("Reciv"),0,5); // Reflect so on display
					received_word *= 5;		// Convert word to frequency in MHz
					received_word += 30875;	// and offset by receive IF
				}
				else if (received_word < 30000)	// This means VHF Transmit
				{
					transmitting = TRUE;		// Make note we are transmitting
					LCDsendFLASHString(PSTR(" 0:00"),0,5);	// Clear timer text
					second_fractions = 0;
					seconds = 0;				// Clear all the timer variables
					minutes = 0;
					received_word *= 5;		// Convert word to frequency in MHz
				}
				else if (received_word < 85000)	// This means UHF Receive
				{
					transmitting = FALSE;	// Make note we're not transmiting 
					LCDsendFLASHString(PSTR("Reciv"),0,5); // Reflect so on display
					received_word *= 5;		// Convert word to frequency in MHz
					received_word += 30875;	// and offset by receive IF
				}
				else 								// UHF Transmit
				{
					transmitting = TRUE;		// Make note we are transmitting
					LCDsendFLASHString(PSTR(" 0:00"),0,5);	// Clear timer text
					second_fractions = 0;
					seconds = 0;				// Clear all the timer variables
					minutes = 0;
					received_word *= 5;		// Convert word to frequency in MHz
				}

				ASCIIUnsigned (tempstr, received_word, 2);	// Convert to text
//				SerTxByte(0x0A,0);
//				SerTxByte(0x0D,0);
//				SerTxByte('f',0);				// Send freq. to serial port for debug
//				SerTxByte('=',0);
//				SerTxString(tempstr, 10, 0);
				LCDsendString((tempstr + 4), 6, 3);		// Three digits of MegaHertz
				LCDsendFLASHString(PSTR("."),9,1);		// Decimal
				LCDsendString((tempstr + 7), 10, 3);	// Three digits of KiloHertz
				LCDsendFLASHString(PSTR("MHz"),13,3);	// Units
				LCDRedraw();					// And tell code to redraw screen
			}		// End if (bytes == 3)
			
			bytes = 0;							// Forget received bytes event
			received_word = 0;				// And prepare next word to be received
		}		// End if (!(TCNT0))

		// If we're transmitting AND the seconds have changed since last time
		if ((transmitting) && (seconds != last_seconds))
		{
			LCDsendFLASHString(PSTR(" "), 0, 1);	// Colon
			ASCIIUnsigned (tempstr, minutes, 0);	// Convert minutes to text
			LCDsendString((tempstr + 2), 1, 1);		// Show minutes digit
			LCDsendFLASHString(PSTR(":"), 2, 1);	// Colon
			ASCIIUnsigned (tempstr, seconds, 0);	// Convert seconds to text
			LCDsendString((tempstr + 1), 3, 2);		// Show seconds digits
			LCDRedraw();						// And tell code to redraw screen
			last_seconds = seconds;			// Remember current seconds value
		}

		LCDProcesses();				// See if the LCD code needs to do anything
		SerProcesses();				// See if the Serial code needs to do anything

	}		// End while(TRUE)

	return(1);

}		// End main(...)


/******************************************************************************/
SIGNAL(TIMER0_OVF_vect)
/*******************************************************************************
* ABSTRACT:	TCNT0 is decremented at 31.25 kHz. 
*				This interrupt clears Idle 5ms after TCNT0 is set to 100.
*/
{
//	TCNT0 = 0;									// Preload for 200Hz
//	active = FALSE;							// Clear Idle flag
//	TIMSK &= ~(1<<TOIE0);					// Disable Timer0 interrupt
	
}		// End SIGNAL(TIMER0_OVF_vect)


/******************************************************************************/
SIGNAL(TIMER1_CAPT_vect)
/*******************************************************************************
* ABSTRACT:	This interrupt is called on the rising endge of ICP1.
*				Here, we take the value of the data line and shift it
*				Into the receive buffer.
*
* PARAMETERS:	None.
*/
{
	if (PINB & 0x02)							// If the data pin is high
	{
		received_byte |= data_bit;			// Save bit in received data byte
	}
	
	data_bit >>= 1;							// Next bit received will be to the right

}		// End SIGNAL(TIMER1_CAPT_vect)


/******************************************************************************/
SIGNAL(TIMER2_OVF_vect)
/*******************************************************************************
* ABSTRACT:	TCNT2 is decremented at 7.8125 kHz. 
*				After 156 counts, it's 1/50.08 seconds... 
*				So load TCNT2 with 100 for roughly 50 Hz.
*/
{
	TCNT2 = 100;								// Preload for ~50Hz
	if (++second_fractions == 52)			// Empirically, it's 52
	{
		second_fractions = 0;				// Reset for next second timeout
		if (++seconds == 60)					// Advance seconds
		{
			seconds = 0;						// Then wrap and update minutes.
			minutes++;
		}

	}
	
}		// End SIGNAL(TIMER2_OVF_vect)


