/*******************************************************************************
File:			Serial.c

Abstract:	Serial I/O function library.

Platform:	ATmega8

Functions:
extern void	SerInit(void);
extern void	SerTxByte(uint8_t chr, char port);
extern void SerTxString(uint8_t *address, char chars, char port);
extern void SerTxFLASHString(const char *addr, char chars, char port);
extern void SerTxEEPROMString(unsigned short addr, char chars, char port);
extern void SerProcesses (void);
SIGNAL(SIG_UART_RECV)
SIGNAL(SIG_UART_TRANS)

Version:		2.00	07/02/16	GND	Gary N. Dion

*******************************************************************************/

// OS headers
#include <avr/io.h>
#include <avr/eeprom.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>

// General purpose include files
#include "StdDefines.h"

// App required include files
#include "Serial.h"

// Local variables
uint8_t in0buf[RX_BUF_SIZE0];				// USART input buffer
uint8_t in0head;								// USART input buffer head pointer
uint8_t in0tail;								// USART input buffer tail pointer
uint8_t out0buf[TX_BUF_SIZE0];			// USART output buffer
uint8_t out0head;								// USART output buffer head pointer
uint8_t out0tail;								// USART output buffer tail pointer

#ifdef SOFTUART
	uint8_t in1buf[RX_BUF_SIZE1];			// SoftUART input buffer
	uint8_t in1head;							// SoftUART input buffer head pointer
	uint8_t in1tail;							// SoftUART input buffer tail pointer
	uint8_t out1buf[TX_BUF_SIZE1];		// SoftUART output buffer
	uint8_t out1head;							// SoftUART output buffer head pointer
	uint8_t out1tail;							// SoftUART output buffer tail pointer
	// The data directions for the following pins get defined in Control.c
	#define	RXPIN			(PIND & 0x20)		// Port D, pin 5 is the RX pin
	#define	TXPINLOW		(PORTD &= ~0x40)	// Port D, pin 6 is the TX pin
	#define	TXPINHIGH	(PORTD |= 0x40)	// Port D, pin 6 is the TX pin
#endif


/******************************************************************************/
extern void	SerInit(void)
/*******************************************************************************
* ABSTRACT:	This function initializes the serial USART.
*				It establishes the baud rate, data and stop bits, and parity.
*
* INPUT:		None
* RETURN:	Void
*/
{
	// Set baud rate
	UBRRH = (uint8_t)(MYUBRR>>8);
	UBRRL = (uint8_t)(MYUBRR);
	
	// Set frame format to 8 data bits, no parity, and 1 stop bit
	UCSRC = (1<<URSEL)|(3<<UCSZ0);

	// Enable the receiver, transmitter, and their respective interrupts
	UCSRB = (1<<RXEN)|(1<<TXEN)|(1<<RXCIE)|(1<<TXCIE);
//	UCSRB = (1<<TXEN)|(1<<TXCIE);

	in0head = in0tail = out0head = out0tail = 0;	// Clear all indices
#ifdef SOFTUART
	in1head = in1tail = out1head = out1tail = 0;	// Clear all the pointers
#endif

	SerTxByte(0x0D,0);
	SerTxByte(0x0A,0);
	SerTxByte('>',0);

	return;

}		// End SerInit(...)


/******************************************************************************/
extern void	SerTxByte(uint8_t chr, uint8_t port)
/*******************************************************************************
* ABSTRACT:	This function pushes a new character into the output buffer,
*				then pre-advances the pointer to the next empty location.
*				It also feeds the UDR if no transmission is already in progress.
*
* INPUT:		chr	byte to Tx
*				port	0 is the USART, 1 is the Software UART
* RETURN:	Void
*/
{
	if (port == 0)								// Port 0 is the USART
	{
		if (++out0head == TX_BUF_SIZE0) out0head = 0;// Advance/wrap pointer

		out0buf[out0head] = chr;		 	// Transfer the byte to output buffer

		if (UCSRA & (1<<UDRE))			// If the transmit register is empty
		{
			if (++out0tail == TX_BUF_SIZE0) out0tail = 0;// Advance/wrap pointer

			UDR = out0buf[out0tail];		// Place byte in the transmit register
		}

	}
#ifdef SOFTUART
	else											// Port 1 is the software UART
	{
		if (++out1head == TX_BUF_SIZE1) out1head = 0;	// Advance/wrap pointer

		out1buf[out1head] = chr;		 	// Transfer the byte to output buffer
	}
#endif

	return;

}		// End TxByte(...)


/******************************************************************************/
extern void SerTxString(uint8_t *address, uint8_t chars, uint8_t port)
/*******************************************************************************
* ABSTRACT:	This function sends a null-terminated string to the serial port.
*
* INPUT:		*address	Pointer to string to send
*				chars		Maximum number of characters to transmit
*				port		0 is the USART, 1 is the Software UART
* RETURN:	Void
*/
{
	while ((*address != 0) && (chars--))// While not end of string
	{
		SerTxByte(*(address++),port);		// Send the byte and increment index
	}

	return;

}		// End SerTxString(...)


/******************************************************************************/
extern void SerTxFLASHString(const uint8_t *addr, uint8_t chars, uint8_t port)
/*******************************************************************************
* ABSTRACT:	This function sends a null-terminated string located in program
*				space to the serial port.
*
* INPUT:		addr		Starting address for the string
*				chars		Maximum number of characters to transmit
*				port		0 is the USART, 1 is the Software UART
* RETURN:	Void
*/
{
#ifdef GND
	uint8_t temp_char;

	temp_char = pgm_read_byte(addr++);	// Preload first character
	while ((temp_char) && (chars--))		// While not end of string
	{
		SerTxByte(temp_char, port);		// Send byte to desired port
		temp_char = pgm_read_byte(addr++);	// Grab next byte
	}
#endif
	return;

}		// End SerTxFLASHString(...)


/******************************************************************************/
extern void SerTxEEPROMString(uint16_t addr, uint8_t chars, uint8_t port)
/*******************************************************************************
* ABSTRACT:	This function Txs a null-terminated string found in EEPROM
*				at the address given to the serial port.
*
* INPUT:		addr		Starting address for the string
*				chars		Maximum number of characters to Send
*				port		0 is the USART, 1 is the Software UART
* RETURN:	Void
*/
{
#ifdef GND
	uint8_t temp_char;

	temp_char = eeprom_read_byte ((uint8_t *)(addr));	// Preload first character
	while ((temp_char) && (chars--))		// While not end of string
	{
		SerTxByte(temp_char, port);		// Tx byte to desired port
		temp_char = eeprom_read_byte ((uint8_t *)(++addr));	// Grab next byte
	}
#endif
	return;

}		// End SerTxEEPROMString(...)


/******************************************************************************/
extern void SerProcesses(void)
/*******************************************************************************
* ABSTRACT:	Called by main.c during idle time, this processes any waiting serial
*				characters coming in either serial port.
*
* INPUT:		None
* RETURN:	Void
*/
{
	if (in0tail != in0head)					// If incoming bytes are pending
	{
		if (++in0tail == RX_BUF_SIZE0) in0tail = 0;// Advance/wrap pointer
//		SerTxByte(in0buf[in0tail]+1,0);	// Echo character for debug
//		MsgHandler0(in0buf[in0tail]);		// And pass it to a handler
	}

#ifdef SOFTUART
	if (in1tail != in1head)					// If incoming bytes are pending
	{
		if (++in1tail == RX_BUF_SIZE1) in1tail = 0;// Advance/wrap pointer
//		SerTxByte(in1buf[in1tail],1);		// Echo character for debug
//		MsgHandler1(in1buf[in1tail]);		// And pass it to a handler
	}
#endif

	return;

}		// End Serial_Processes(...)


#ifdef SOFTUART
/******************************************************************************/
extern void	SerSoftUART_TX(void)
/*******************************************************************************
* ABSTRACT:	This function is called at the desired baud rate for the transmit
*				port.  It effectively creates a software UART transmitter using the
*				pins declared at the top of this file.  Protocol is 8N1.
*
* INPUT:		None
* RETURN:	Void
*/
{
	static uint8_t txbit = 10;		// 1 start bit, 8 data, and 1 stop bit
	static uint8_t tx_byte;			// Holds the byte to transmit

	if (txbit == 10)							// If the transmitter is idle
	{
		if (out1tail != out1head)			// And if a least one tx byte is pending
		{
			if (++out1tail == TX_BUF_SIZE1) out1tail = 0;// Advance/wrap pointer

			tx_byte = out1buf[out1tail];	// Take the byte from the output buffer
			txbit = 0;							// Set bit index to zero
		}

	}

	if (txbit != 10)							// A byte is being transmitted
	{
		if (!txbit)								// First bit?
		{
			TXPINLOW;							// First bit is the start bit
		}
		else										// Not the first bit
		{
			if (txbit != 9)					// For each of the 8-bits in the byte
			{
				if (tx_byte & 1) 
				{
					TXPINHIGH;// If bit is high, pin is high
				}
				else 
				{
					TXPINLOW;					// else, pin is low
				}

				tx_byte >>= 1;					// Shift byte right for next bit
			}
			else
			{
				TXPINHIGH;						// Send a high for the stop bit
			}

		}

		txbit++;									// Increment bit count for next time
	}

	return;

}		// End SerSoftUART_TX(...)


/******************************************************************************/
extern void	SerSoftUART_RX(void)
/*******************************************************************************
* ABSTRACT:	This function must be called at a rate of 4 times the desired baud
*				rate.  This function effectively creates a software UART using the
*				pins declared at the top of this file.  Protocol is 8N1.
*
* INPUT:		None
* RETURN:	Void
*/
{
	static uint8_t rxbit = 9;
	static uint8_t rxclock;
	static uint8_t rx_byte;

	if (rxbit == 9)							// Not receiving, seeking the start bit
	{
		if (RXPIN == 0) 						// A low value may be a start bit
		{
			if (rxclock == 0)					// Cleared by previous start loop
			{
				rx_byte = 0;					// Begin with a clean slate
				rxbit = 0;						// Bit index is lsb first
			}

			rxclock = 0;						// Always reset the receive clock
		}

	}
	else
	{
		if (!(++rxclock % 4))				// Execute code every 4th rx clock tick
		{
			if (rxbit != 8)					// Only grab eight data bits
			{
				if (RXPIN) rx_byte |= 1<<rxbit;	// Stuff next bit in receive byte

			}
			else									// We have eight bits now
			{
				if (RXPIN)						// Make sure bit is high for stop bit
				{
					if (++in1head == RX_BUF_SIZE1) in1head = 1;// Advance & wrap

					in1buf[in1head] = rx_byte;	// Place byte in receive buffer
				}

			}

			rxbit++;								// Advance receive bit counter
		}

	}

}		// End SerSoftUART_RX(...)

#endif


/******************************************************************************/
SIGNAL(USART_RXC_vect)
/*******************************************************************************
* ABSTRACT:	Called by the receive ISR (interrupt). Saves the next serial byte
*				to the head of the RX buffer.
*/
{
	if (++in0head == RX_BUF_SIZE0) in0head = 0;	// Advance/wrap pointer

	in0buf[in0head] = UDR;	  				// Transfer the byte to the input buffer

}		// End SIGNAL(USART_RXC_vect)


/******************************************************************************/
SIGNAL(USART_TXC_vect)
/*******************************************************************************
* ABSTRACT:	Called by the transmit ISR (interrupt). Puts the next serial byte
*				into the TX register.
*/
{
	if (out0tail != out0head)				// If there are outgoing bytes pending
	{
		if (++out0tail == TX_BUF_SIZE0) out0tail = 0;	// Advance/wrap pointer

		UDR = out0buf[out0tail];			// Place the byte in the output register
	}

}		// End SIGNAL(USART_TXC_vect)
